@@ -12,8 +12,9 @@ pointNormalOnLine <- function(pt, l1, l2=NULL){
 	r <- matrix(NA, nrow=nrow(pt), ncol=ncol(pt))
 
 	for(i in 1:nrow(pt)){
+
 		# IF L2 ARGUMENT IS NULL THEN TREAT L1 AS VECTOR AND GET SECOND POINT ON LINE THROUGH ORIGIN
-		if(is.null(l2)) l2 <- 2*l1
+		if(is.null(l2)) l2 <- c(0,0,0)
 	
 		# CHECK THAT POINTS DEFINING LINE ARE NOT COINCIDENT
 		if(sum((l2 - l1)^2) == 0) return(NA)
